#include "ImageViewer.h"

#include <QIcon>
#include <QList>
#include <QImage>
#include <QString>
#include <QPainter>
#include <QMessageBox>

struct ImageViewerData
{
	QString fileName;
	QImage image;
	QPointF* pixelPoint;
	QPolygonF polygon;

	int numPoints;
	bool drawPoly;

	QList<QPointF> pathPoints;
};

ImageViewer::ImageViewer(QString fileName, QWidget* parent) 
: QWidget(parent)
{
	setWindowIcon(QIcon("logosmall.jpg"));
	setWindowTitle("Region Growing Application");

	d = new ImageViewerData;
	d->fileName = fileName;
	d->image = QImage(d->fileName);

	// Needed to store the list of points
	d->numPoints = 0;
	d->drawPoly = false;

	if(d->image.isNull())
	{
		QString msg = QString("%1:%2").arg("Invalid Image", d->fileName);
		QMessageBox::warning(0, "Error01", msg);
		return;
	}
}

ImageViewer::~ImageViewer()
{
	delete d;
}

void ImageViewer::mouseMoveEvent(QMouseEvent* me)
{
	update();
}

void ImageViewer::mousePressEvent(QMouseEvent* me)
{
	// If Mouse right button, then just update polygon list and draw
	if(me->button() == Qt::RightButton)
	{
		d->drawPoly = true;
		updatePolygon();

		return;
	}
	d->drawPoly = false;

	// Done so that always a new polygon list is created
	d->polygon.clear();
	d->pathPoints.append(me->posF());

	update();
}

void ImageViewer::mouseReleaseEvent(QMouseEvent* me)
{
	update();
}

void ImageViewer::paintEvent(QPaintEvent* pe)
{
	QPainter p(this);

	p.drawImage( QRectF(rect()), d->image ); 

	// This draws all clicked points
	p.setPen(Qt::red);
	for(int i=0; i<d->pathPoints.count(); i++)
		p.drawPoint(d->pathPoints[i]);

	// Only draw the polygon if right click
	if(d->drawPoly == true)
		p.drawPolygon(d->polygon);
}

void ImageViewer::updatePolygon()
{
	for(int i=0; i<d->pathPoints.count(); i++)
	{
		d->polygon << d->pathPoints[i];
	}

	// Done polygon drawing so clear the path points buffer
	d->pathPoints.clear();
}